﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using System.Collections;
using Infragistics.Controls.Charts;

namespace OlapAxisSample
{
    public class CarSales
    {
        public Car Car { get; set; }
        public SalesRegion SalesRegion { get; set; }
        public DateTime SalesDate { get; set; }
        public int SoldNewUnits { get; set; }
        public int SoldUsedUnits { get; set; }
        public int SoldTotalUnits { get; set; }
    }

    public class Car
    {
        public string Brand { get; set; }
        public string Model { get; set; }
        public int Year { get; set; }
    }

    public class SalesRegion
    {
        public string Continent { get; set; }
        public string Country { get; set; }
        public string City { get; set; }
    }

    public partial class MainPage : UserControl
    {
        private Random _random = new Random();
        
        private Dictionary<string, List<string>> _brandsModels = new Dictionary<string,List<string>>
        {
            {"Audi", new List<string> { "A3", "A4", "A5", "A6" }},
            {"BMW", new List<string> { "3 Series", "5 Series", "6 Series", "7 Series" }},
            {"Mercedes-Benz", new List<string> { "A Class", "B Class", "C Class", "S Class" }},
            {"Peugeot", new List<string> { "206", "306", "307", "308" }},
            {"Porsche", new List<string> { "911 Carrera", "911 Carrera S", "Panamera", "Cayenne" }},
        };

        private List<SalesRegion> _regions = new List<SalesRegion>
        {
            new SalesRegion
            {
                City = "Sofia",
                Continent = "Europe",
                Country = "Bulgaria"
            },
            new SalesRegion
            {
                City = "Bucharest",
                Continent = "Europe",
                Country = "Romania"
            },
            new SalesRegion
            {
                City = "Berlin",
                Continent = "Europe",
                Country = "Germany"
            },
            new SalesRegion
            {
                City = "San Francisco",
                Continent = "North America",
                Country = "USA"
            },
            new SalesRegion
            {
                City = "Los Angeles",
                Continent = "North America",
                Country = "USA"
            },
            new SalesRegion
            {
                City = "Rio de Janeiro",
                Continent = "South America",
                Country = "Brazil"
            },
        };

        private long LongRandom(long min, long max)
        {
            byte[] buf = new byte[8];
            _random.NextBytes(buf);
            long longRand = BitConverter.ToInt64(buf, 0);

            return (Math.Abs(longRand % (max - min)) + min);
        }

        private T SelectRandomItem<T>(IEnumerable<T> list)
        {
            int itemIndex = (int)LongRandom(0, list.Count());
            return list.ElementAt(itemIndex);
        }
        
        public MainPage()
        {
            InitializeComponent();
            ((OlapAxis)DataChart.Axes["olapXAxis"]).SeriesCreating += 
                new EventHandler<SeriesCreatingEventArgs>(olapXAxis_SeriesCreating);
            ((OlapAxis)DataChart.Axes["olapXAxis"]).SeriesInitializing += 
                new EventHandler<SeriesInitializingEventArgs>(olapXAxis_SeriesInitializing);
        }

        private void olapXAxis_SeriesCreating(object sender, 
            SeriesCreatingEventArgs e)
        {
            return;
            if (e.SeriesInfo.Title == "SoldNewUnits")
            {
                e.Series = new LineSeries { ValueMemberPath = e.SeriesInfo.Title };
            }
            else
            {
                e.Series = new PointSeries { ValueMemberPath = e.SeriesInfo.Title };
            }
        }

        private void olapXAxis_SeriesInitializing(object sender, 
            SeriesInitializingEventArgs e)
        {
        }

        private CarSales CreateRandomCarSale()
        {
            int newUnits = (int)LongRandom(1000, 50000);
            int usedUnits = (int)LongRandom(1000, 100000);
            return new CarSales
            {
                Car = CreateRandomCar(),
                SalesDate = SelectRandomDate(),
                SalesRegion = SelectRandomSalesRegion(),
                SoldNewUnits = newUnits,
                SoldUsedUnits = usedUnits,
                SoldTotalUnits = newUnits + usedUnits
            };
        }

        private Car CreateRandomCar()
        {
            var brandModel = SelectRandomBrandModel();
            return new Car
            {
                Brand = brandModel.Item1,
                Model = brandModel.Item2,
                Year = SelectRandomYear()
            };
        }

        private DateTime SelectRandomDate()
        {
            DateTime start = new DateTime(2000, 1, 1);
            DateTime end = new DateTime(2010, 1, 1);

            long randomDateTics = LongRandom(start.Ticks, end.Ticks);

            return new DateTime(randomDateTics);
        }

        private SalesRegion SelectRandomSalesRegion()
        {
            return SelectRandomItem(_regions);
        }

        private Tuple<string, string> SelectRandomBrandModel()
        {
            string brand = SelectRandomItem(_brandsModels.Keys);
            string model = SelectRandomItem(_brandsModels[brand]);
            return Tuple.Create(brand, model);
        }

        private int SelectRandomYear()
        {
            return (int)LongRandom(1990, 2005);
        }

        private List<CarSales> _data;
        public List<CarSales> Data
        {
            get
            {
                if (_data == null)
                {
                    _data = new List<CarSales>();
                    for (int i = 0; i < 100; i++)
                    {
                        _data.Add(CreateRandomCarSale());
                    }
                }
                return _data;
            }
        }

        public IEnumerable<Tuple<string, int, int, int>> GroupedData
        {
            get
            {
                return Data
                    .GroupBy(cs => cs.Car.Brand)
                    .Select(g =>
                        Tuple.Create(g.Key,
                            g.Sum(cs => cs.SoldNewUnits),
                            g.Sum(cs => cs.SoldTotalUnits),
                            g.Sum(cs => cs.SoldUsedUnits)));
            }
        }
    }
}
